procedure Image.DrawRotated(X,Y: Int64; Deg: Int64);
var
	tX, tY: LongInt;
	
begin
	PositionX := X;
	PositionY := Y;
	
	tX := GlWidth div 2;
	tY := GlHeight div 2;

	glBindTexture(GL_TEXTURE_2D,Texture);
	
	glPushMatrix();  // Save modelview matrix

	glTranslatef(PositionX,PositionY,0.0);  // Position sprite
	glRotatef(Deg,0,0,1);

	glBegin(GL_QUADS);                                   // Draw sprite 
	glTexCoord2f(0.0,1); 		glVertex3i( trunc(-tX / XScale), trunc(tY / YScale),0);
	glTexCoord2f(0.0,0.0); 		glVertex3i( trunc(-tX / XScale), trunc(-tY / YScale),0);
	glTexCoord2f(1,0.0); 		glVertex3i( trunc(tX / XScale), trunc(-tY / Yscale),0);
	glTexCoord2f(1,1); 			glVertex3i( trunc(tX / XScale), trunc(tY / Yscale),0);
	glEnd();

	glPopMatrix();  // Restore modelview matrix
	Rotation := Deg;
end;

procedure Image.Rotate(Deg: Int64);
begin
	if PrometheusVideo.RotationMode = Mode_Absolute then
		Rotation := Deg
	else if PrometheusVideo.RotationMode = Mode_Incremental then
		Rotation := Rotation + Deg;
end;

function Image.Width(): Int64;
begin
	Width := GlWidth;
end;

function Image.Height(): Int64;
begin
	Height := GlHeight;
end;

procedure Image.Draw(X,Y: Int64);
var
	tx, ty: LongInt;
	
begin
	PositionX := X;
	PositionY := Y;
	
	tx := GlWidth div 2;
	ty := GlHeight div 2;
	
	if Rotation <> 0 then
		begin
			DrawRotated(X, Y, Rotation);
		end
	else
		begin
			glBindTexture( GL_TEXTURE_2D, texture );

			glBegin( GL_QUADS );
				glTexCoord2f(0.0,1); 		glVertex3i( trunc(-tX / XScale), trunc(tY / YScale),0);
				glTexCoord2f(0.0,0.0); 		glVertex3i( trunc(-tX / XScale), trunc(-tY / YScale),0);
				glTexCoord2f(1,0.0); 		glVertex3i( trunc(tX / XScale), trunc(-tY / Yscale),0);
				glTexCoord2f(1,1); 			glVertex3i( trunc(tX / XScale), trunc(tY / Yscale),0);
			glEnd();
		end;
end;


procedure Image.LoadFromSurface(Src: pSdl_Surface);
begin
	if Assigned(Src) then
		begin			
			glGenTextures(1, @Texture);
			glBindTexture(GL_TEXTURE_2D, Texture);
		 
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		 
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Src^.w, Src^.h, 0, GL_RGBA, GL_UNSIGNED_BYTE, Src^.pixels);
		end;
    
    XScale := 1;
    YScale := 1;
    
    GlWidth := src^.W;
    GlHeight := Src^.H;
    Sdl_FreeSurface(Src);
end;

procedure Image.Load(Source: String);
var
	Surface: pSdl_Surface;
	
begin
    Surface := Img_load( Pchar(Source) );
    
    NOfcolors := Surface.Format.BytesperPixel;
    
    if NOfColors = 4 then
	begin
	    if Surface.Format.Rmask = $000000ff then
		Texture_Format := Gl_RGBA;
	end
    else if NOfColors = 3 then
	begin
	    if Surface.Format.Rmask = $000000ff then
		Texture_Format := Gl_RGB;
	end
    else
	begin
	end;
    
    glGenTextures( 1, @Texture );
    
    glBindTexture( GL_TEXTURE_2D, texture );
    
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    
    glTexImage2D( GL_TEXTURE_2D, 0, nOfColors, surface.w, surface.h, 0, texture_format, GL_UNSIGNED_BYTE, surface.pixels );
    
    //Sdl_FreeSurface(Surface);
    
    XScale := 1;
    YScale := 1;
    
    GlWidth := surface.W;
    GlHeight := surface.H;
    
    Sdl_FreeSurface(Surface);
end;

procedure Image.Empty();
begin
	//sdl_freesurface(Surface);
	NOfcolors := 0;
	XScale := 0;
	YScale := 0;
	GlWidth := 0;
	GlHeight := 0;
	Texture_Format := 0;
	glDeleteTextures(1, @Texture);
end;

procedure Image.DrawSelection(sX, sY, sW, sH, dX, dY: Int64);
var
    VxR: array [1..2] of real;
    VyR: array [1..2] of real;
    
    PxR: array [1..2] of Int64;
    PyR: array [1..2] of Int64;
    
begin
    VxR[1] := sX / GlWidth;
    VxR[2] := (sX + sW) / GlWidth;
    VyR[1] := sY / GlHeight;
    VyR[2] := (sY + sH) / GlHeight;
    
    PxR[1] := dX;
    PxR[2] := dX + sW;
    PyR[1] := dY;
    PyR[2] := dY + sH;

    glBindTexture( GL_TEXTURE_2D, texture );

    glBegin( GL_QUADS );
		glTexCoord2f( VxR[1], VyR[1] );
		glVertex3f( PxR[1], PyR[1], 0. );

		glTexCoord2f( VxR[2], VyR[1] );
		glVertex3f( PxR[2], PyR[1], 0. );

		glTexCoord2f( VxR[2], VyR[2] );
		glVertex3f( PxR[2], PyR[2], 0. );

		glTexCoord2f( VxR[1], VyR[2] );
		glVertex3f( PxR[1], PyR[2], 0. );
    glEnd();
end;

procedure Image.Resize(X,Y: Int64); 
begin
	XScale := GlWidth / X;
	YScale := GlHeight / Y;
end;
